@@ -51,8 +51,6 @@ import org.apache.tomcat.util.res.StringManager;
  * <li>tempDir - a temporaryDirectory to store binary data when downloading a war from the cluster</li>
  * </ul>
  * Currently we only support deployment of WAR files since they are easier to send across the wire.
- *
- * @author Peter Rossbach
  */
 public class FarmWarDeployer extends ClusterListener implements ClusterDeployer, FileChangeListener {
     /*--Static Variables----------------------------------------*/
@@ -146,8 +144,7 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
             return;
         }
         Engine engine = (Engine) econtainer;
-        String hostname = null;
-        hostname = host.getName();
+        String hostname = host.getName();
         try {
             oname = new ObjectName(engine.getName() + ":type=Deployer,host=" + hostname);
         } catch (Exception e) {
@@ -164,7 +161,7 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
         configBase = host.getConfigBaseFile();
 
         // Retrieve the MBean server
-        mBeanServer = Registry.getRegistry(null, null).getMBeanServer();
+        mBeanServer = Registry.getRegistry(null).getMBeanServer();
 
         started = true;
         count = 0;
@@ -238,8 +235,8 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                         } else {
                             log.error(sm.getString("farmWarDeployer.servicingDeploy", contextName, name));
                         }
-                    } catch (Exception ex) {
-                        log.error(sm.getString("farmWarDeployer.fileMessageError"), ex);
+                    } catch (Exception e) {
+                        log.error(sm.getString("farmWarDeployer.fileMessageError"), e);
                     } finally {
                         removeFactory(fmsg);
                     }
@@ -263,12 +260,12 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                     } else {
                         log.error(sm.getString("farmWarDeployer.servicingUndeploy", contextName));
                     }
-                } catch (Exception ex) {
-                    log.error(sm.getString("farmWarDeployer.undeployMessageError"), ex);
+                } catch (Exception e) {
+                    log.error(sm.getString("farmWarDeployer.undeployMessageError"), e);
                 }
             }
-        } catch (IOException x) {
-            log.error(sm.getString("farmWarDeployer.msgIoe"), x);
+        } catch (IOException ioe) {
+            log.error(sm.getString("farmWarDeployer.msgIoe"), ioe);
         }
     }
 
@@ -303,13 +300,9 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
     }
 
     /**
-     * Before the cluster invokes messageReceived the cluster will ask the receiver to accept or decline the message, In
-     * the future, when messages get big, the accept method will only take a message header
-     *
-     * @param msg ClusterMessage
-     *
-     * @return boolean - returns true to indicate that messageReceived should be invoked. If false is returned, the
-     *             messageReceived method will not be invoked.
+     * {@inheritDoc}
+     * <p>
+     * This listener accepts only FileMessage or UndeployMessage.
      */
     @Override
     public boolean accept(ClusterMessage msg) {
@@ -400,18 +393,13 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                     log.error(sm.getString("farmWarDeployer.removeFailRemote", contextName));
                 }
 
-            } catch (Exception ex) {
-                log.error(sm.getString("farmWarDeployer.removeFailLocal", contextName), ex);
+            } catch (Exception e) {
+                log.error(sm.getString("farmWarDeployer.removeFailLocal", contextName), e);
             }
         }
 
     }
 
-    /**
-     * Modification from watchDir war detected!
-     *
-     * @see org.apache.catalina.ha.deploy.FileChangeListener#fileModified(File)
-     */
     @Override
     public void fileModified(File newWar) {
         try {
@@ -438,16 +426,11 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                 log.error(sm.getString("farmWarDeployer.servicingDeploy", cn.getName(), deployWar.getName()));
             }
             install(cn.getName(), deployWar);
-        } catch (Exception x) {
-            log.error(sm.getString("farmWarDeployer.modInstallFail"), x);
+        } catch (Exception e) {
+            log.error(sm.getString("farmWarDeployer.modInstallFail"), e);
         }
     }
 
-    /**
-     * War remove from watchDir
-     *
-     * @see org.apache.catalina.ha.deploy.FileChangeListener#fileRemoved(File)
-     */
     @Override
     public void fileRemoved(File removeWar) {
         try {
@@ -456,8 +439,8 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                 log.info(sm.getString("farmWarDeployer.removeLocal", cn.getName()));
             }
             remove(cn.getName(), true);
-        } catch (Exception x) {
-            log.error(sm.getString("farmWarDeployer.removeLocalFail"), x);
+        } catch (Exception e) {
+            log.error(sm.getString("farmWarDeployer.removeLocalFail"), e);
         }
     }
 
@@ -502,7 +485,7 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
      */
     protected void undeployDir(File dir) {
 
-        String files[] = dir.list();
+        String[] files = dir.list();
         if (files == null) {
             files = new String[0];
         }
@@ -586,16 +569,6 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
     }
 
     /*--Instance Getters/Setters--------------------------------*/
-    @Override
-    public boolean equals(Object listener) {
-        return super.equals(listener);
-    }
-
-    @Override
-    public int hashCode() {
-        return super.hashCode();
-    }
-
     public String getDeployDir() {
         return deployDir;
     }
@@ -706,8 +679,8 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                     return false;
                 }
             }
-        } catch (IOException e) {
-            log.error(sm.getString("farmWarDeployer.fileCopyFail", from, to), e);
+        } catch (IOException ioe) {
+            log.error(sm.getString("farmWarDeployer.fileCopyFail", from, to), ioe);
             return false;
         }
 
@@ -721,8 +694,8 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
                 }
                 os.write(buf, 0, len);
             }
-        } catch (IOException e) {
-            log.error(sm.getString("farmWarDeployer.fileCopyFail", from, to), e);
+        } catch (IOException ioe) {
+            log.error(sm.getString("farmWarDeployer.fileCopyFail", from, to), ioe);
             return false;
         }
         return true;
@@ -745,7 +718,8 @@ public class FarmWarDeployer extends ClusterListener implements ClusterDeployer,
         }
         try {
             dir = dir.getCanonicalFile();
-        } catch (IOException e) {// ignore
+        } catch (IOException ignore) {
+            // Ignore
         }
         return dir;
     }
