@@ -375,8 +375,8 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                     try {
                         // Validate and write response headers
                         prepareResponse();
-                    } catch (IOException e) {
-                        handleIOException(e);
+                    } catch (IOException ioe) {
+                        handleIOException(ioe);
                     }
                 }
                 break;
@@ -385,8 +385,8 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                 action(ActionCode.COMMIT, null);
                 try {
                     finishResponse();
-                } catch (IOException e) {
-                    handleIOException(e);
+                } catch (IOException ioe) {
+                    handleIOException(ioe);
                 }
                 break;
             }
@@ -397,8 +397,8 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             case EARLY_HINTS: {
                 try {
                     earlyHints();
-                } catch (IOException e) {
-                    handleIOException(e);
+                } catch (IOException ioe) {
+                    handleIOException(ioe);
                 }
                 break;
             }
@@ -406,9 +406,9 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                 action(ActionCode.COMMIT, null);
                 try {
                     flush();
-                } catch (IOException e) {
-                    handleIOException(e);
-                    response.setErrorException(e);
+                } catch (IOException ioe) {
+                    handleIOException(ioe);
+                    response.setErrorException(ioe);
                 }
                 break;
             }
@@ -1075,7 +1075,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
         // information (e.g. client IP)
         setSocketWrapper(socketWrapper);
         // Set up the minimal request information
-        request.setStartTimeNanos(System.nanoTime());
+        request.markStartTime();
         // Set up the minimal response information
         response.setStatus(400);
         response.setError();
